@@ -2,7 +2,9 @@ from datetime import datetime
 from .exceptions import MaxTriesError, RequiresAuthenticationError
 import os
 import requests
-from typing import BinaryIO, Dict, List, Optional
+from typing import Any, BinaryIO, Dict, List, Optional, Union
+
+MOLT_CHARACTER_LIMIT = 280
 
 
 class API:
@@ -21,13 +23,13 @@ class API:
         if access_token:
             self.authenticate(access_token)
 
-    def authenticate(self, access_token: str):
+    def authenticate(self, access_token: str) -> bool:
         self.access_token = access_token
         r = self._make_request('/authenticate/')
         self.crab = self._objectify(r.json(), 'crab')
         return r.ok
 
-    def get_current_user(self):
+    def get_current_user(self) -> 'Crab':
         """ Get the current authenticated user.
         """
         return self.crab
@@ -76,28 +78,47 @@ class API:
             self._molts[molt_id] = None
             return None
 
-    def get_molts_with_crabtag(self, crabtag: str, limit=10, offset=0) \
-            -> List['Molt']:
+    def get_molts_with_crabtag(self, crabtag: str, limit: int = 10,
+                               offset: int = 0, since_ts: Optional[int] = None,
+                               since_id: Optional[int] = None) -> List['Molt']:
         """ Get molts that contain the crabtag `crabtag`.
         """
         r = self._make_request(f'/crabtag/{crabtag}/',
-                               params={'limit': limit, 'offset': offset})
+                               params={'limit': limit, 'offset': offset,
+                                       'since': since_ts,
+                                       'since_id': since_id})
         return [self._objectify(molt, 'molt')
                 for molt in r.json().get('molts', list())]
 
-    def get_molts_mentioning(self, username: str, limit=10, offset=0) \
-            -> List['Molt']:
+    def get_molts_mentioning(self, username: str, limit: int = 10,
+                             offset: int = 0, since_ts: Optional[int] = None,
+                             since_id: Optional[int] = None) -> List['Molt']:
         """ Get Molts that explicitly mention `username`.
         """
         r = self._make_request(f'/molts/mentioning/{username}/',
-                               params={'limit': limit, 'offset': offset})
+                               params={'limit': limit, 'offset': offset,
+                                       'since': since_ts,
+                                       'since_id': since_id})
         return [self._objectify(molt, 'molt')
                 for molt in r.json().get('molts', list())]
 
-    def post_molt(self, content: str, image_path: Optional[str] = None):
+    def get_molts_replying_to(self, username: str, limit: int = 10,
+                              offset: int = 0, since_ts: Optional[int] = None,
+                              since_id: Optional[int] = None) -> List['Molt']:
+        """ Get Molts that reply to Molts submitted by `username`.
+        """
+        r = self._make_request(f'/molts/replying/{username}/',
+                               params={'limit': limit, 'offset': offset,
+                                       'since': since_ts,
+                                       'since_id': since_id})
+        return [self._objectify(molt, 'molt')
+                for molt in r.json().get('molts', list())]
+
+    def post_molt(self, content: str, image_path: Optional[str] = None) \
+            -> Optional['Molt']:
         """ Post new Molt as the authenticated user.
         """
-        if len(content) <= 240:
+        if len(content) <= MOLT_CHARACTER_LIMIT:
             if self.access_token:
                 if image_path:
                     if not os.path.exists(image_path):
@@ -113,15 +134,16 @@ class API:
                 if r.ok:
                     return self._objectify(r.json(), 'molt')
                 else:
-                    return False
+                    return None
             else:
                 raise RequiresAuthenticationError(
                     'You are not properly authenticated for this request.'
                 )
         else:
-            raise ValueError('Molts cannot exceed 240 characters.')
+            raise ValueError(f'Molts cannot exceed {MOLT_CHARACTER_LIMIT} '
+                             'characters.')
 
-    def _check_connection(self):
+    def _check_connection(self) -> bool:
         r = self._make_request()
         if r.ok:
             return True
@@ -130,7 +152,8 @@ class API:
                                   'Is your base_url accurate?')
 
     def _get_paginated_data(self, endpoint: str, data_key: str,
-                            limit: int = 10, starting_offset: int = 0):
+                            limit: int = 10, starting_offset: int = 0) \
+            -> Dict[str, Any]:
         json_data = list()
         offset = starting_offset
         while True:
@@ -201,7 +224,7 @@ class API:
         else:
             raise MaxTriesError('Failed to complete request.')
 
-    def _objectify(self, json: dict, type: str):
+    def _objectify(self, json: dict, type: str) -> Union['Crab', 'Molt']:
         if type.lower() == 'crab':
             id = json['id']
             if id in self._crabs:
@@ -278,8 +301,7 @@ class Bio:
                                         data=new_bio)
         if r.ok:
             self.crab._json = r.json()
-            return True
-        return r
+        return r.ok
 
 
 class Crab:
@@ -358,7 +380,7 @@ class Crab:
     def timestamp(self) -> int:
         return self._json['register_time']
 
-    def follow(self):
+    def follow(self) -> bool:
         """ Follow this Crab as the authenticated user.
         """
         if self.api.access_token:
@@ -369,7 +391,7 @@ class Crab:
             'You are not properly authenticated for this request.'
         )
 
-    def unfollow(self):
+    def unfollow(self) -> bool:
         """ Unfollow this Crab as the authenticated user.
         """
         if self.api.access_token:
@@ -380,15 +402,33 @@ class Crab:
             'You are not properly authenticated for this request.'
         )
 
-    def get_mentions(self, limit: int = 10, offset: int = 0):
+    def get_mentions(self, limit: int = 10, offset: int = 0,
+                     since_ts: Optional[int] = None,
+                     since_id: Optional[int] = None) -> List['Molt']:
         """ Get Molts that mention this user.
         """
         return self.api.get_molts_mentioning(self.username, limit=limit,
-                                             offset=offset)
+                                             offset=offset, since_ts=since_ts,
+                                             since_id=since_id)
 
-    def get_molts(self, limit=10, offset=0) -> List['Molt']:
+    def get_replies(self, limit: int = 10, offset: int = 0,
+                    since_ts: Optional[int] = None,
+                    since_id: Optional[int] = None) -> List['Molt']:
+        """ Get Molts that reply to any of this user's Molts.
+        """
+        return self.api.get_molts_replying_to(self.username, limit=limit,
+                                              offset=offset, since_ts=since_ts,
+                                              since_id=since_id)
+
+    def get_molts(self, limit: int = 10, offset: int = 0,
+                  since_ts: Optional[int] = None,
+                  since_id: Optional[int] = None) -> List['Molt']:
+        """ Get this user's Molts.
+        """
         r = self.api._make_request(f'/crabs/{self.id}/molts/',
-                                   params={'limit': limit, 'offset': offset})
+                                   params={'limit': limit, 'offset': offset,
+                                           'since': since_ts,
+                                           'since_id': since_id})
         return [self.api._objectify(molt, 'molt')
                 for molt in r.json().get('molts', list())]
 
@@ -454,15 +494,19 @@ class Molt:
     def timestamp(self) -> int:
         return self._json['timestamp']
 
-    def get_replies(self, limit=10, offset=0) -> List['Molt']:
+    def get_replies(self, limit: int = 10, offset: int = 0,
+                    since_ts: Optional[int] = None,
+                    since_id: Optional[int] = None) -> List['Molt']:
         """ Get this Molt's replies.
         """
         r = self.api._make_request(f'/molts/{self.id}/replies/',
-                                   params={'limit': limit, 'offset': offset})
+                                   params={'limit': limit, 'offset': offset,
+                                           'since': since_ts,
+                                           'since_id': since_id})
         return [self.api._objectify(molt, 'molt')
                 for molt in r.json().get('molts', list())]
 
-    def like(self):
+    def like(self) -> bool:
         """ Like this Molt as the authenticated user.
         """
         if self.api.access_token:
@@ -473,7 +517,7 @@ class Molt:
             'You are not properly authenticated for this request.'
         )
 
-    def unlike(self):
+    def unlike(self) -> bool:
         """ Unlike this Molt as the authenticated user.
         """
         if self.api.access_token:
@@ -484,7 +528,7 @@ class Molt:
             'You are not properly authenticated for this request.'
         )
 
-    def delete(self):
+    def delete(self) -> bool:
         """ Delete this Molt if the authenticated user is the author.
         """
         if self.api.access_token:
@@ -495,7 +539,7 @@ class Molt:
             'You are not properly authenticated for this request.'
         )
 
-    def remolt(self):
+    def remolt(self) -> bool:
         """ Remolt this Molt if the authenticated user is the author.
         """
         if self.api.access_token:
@@ -506,7 +550,7 @@ class Molt:
             'You are not properly authenticated for this request.'
         )
 
-    def unremolt(self):
+    def unremolt(self) -> bool:
         """ Unremolt this Molt if the authenticated user is the author.
         """
         if self.api.access_token:
@@ -517,7 +561,8 @@ class Molt:
             'You are not properly authenticated for this request.'
         )
 
-    def reply(self, content: str, image_path: Optional[str] = None):
+    def reply(self, content: str, image_path: Optional[str] = None) \
+            -> Optional['Molt']:
         """ Reply to this Molt as the authenticated user.
         """
         if len(content) <= 240:
@@ -537,7 +582,7 @@ class Molt:
                 if r.ok:
                     return self.api._objectify(r.json(), 'molt')
                 else:
-                    return False
+                    return None
             else:
                 raise RequiresAuthenticationError(
                     'You are not properly authenticated for this request.'
